// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package pack

import (
  "fmt"
  "sort"
  "pagure.io/modist/filter"
  "pagure.io/modist/version"
  "pagure.io/modist/write"
)

// Pack the modules found in the tree located at root, into destination,
// prefixed by prefix, attributing them version, applying filters to their file
// lists, reporting the result in output with outputFormat and being quiet
func Scan(sources []string, ver *version.Version,
          includeMods []string, ignoreMods []string, filters []filter.Filter,
          prefix string, destination string,
          output string, outputFormat string, quiet bool,
         ) (written write.Files, err error) {
  written = make(write.Files)
  rawModules, err := filter.Scan(sources, ver, includeMods, ignoreMods, filters,
                                 quiet)
  if err != nil {
    return written, err
  }
  var modules []string
  for module := range rawModules {
    modules = append(modules, module)
  }
  sort.Strings(modules)
  for _, module := range modules {
    if !quiet {
      fmt.Printf(" – packing      %s\n", module)
    }
    err = written.Pack(rawModules[module], prefix, destination, quiet)
    if err != nil {
      return written, err
    }
  }
  if !quiet {
    report, err := written.Format(outputFormat)
    if err != nil {
      return written, err
    }
    fmt.Printf("\nResults:\n%s", report)
  }
  if output != "" {
    err = written.Output(output, outputFormat)
  }
  return written, err
}
